#!/bin/perl
# Unix Field Pack for EPAgent
# Copyright 2004-2006 Wily Technology, Inc. All rights reserved.
# Version 0.3 - 29 December 2006
# Sylvain Gibassier

# Note: This script creates a metric called "netstat:Success"
# The value for this metric is 2 is the script is functioning normally.
# If there is a problem, the value of this metric should be 1 and there
# should be a netstat_errors.log file in the logs directory. This file contains
# some diagnostic information for the problem that this script is experiencing.

use Sys::Hostname;
# warnings and strict commented out for the default version of Perl that comes with Solaris 8
# use warnings;
# use strict;

my $error_log = ".../../logs/netstat_errors.log";

eval {
	my $host = hostname;
	$error_log = "../../logs/netstat_errors.${host}.log";

	my $netstat_command = 'netstat -n -f inet|';

	if ($^O eq 'MSWin32') {
		$netstat_command = '../../netstat_test.txt';
	} elsif ($^O eq 'linux') {
		$netstat_command = 'netstat -nt|';
	}

	open(NETSTAT,$netstat_command) || die "Error when executing \"$netstat_command\": $!";

	my $ESTABLISHED = 0;
	my $SYN_SENT = 0;
	my $SYN_RECV = 0;
	my $FIN_WAIT1 = 0;
	my $FIN_WAIT2 = 0;
	my $TIME_WAIT = 0;
	my $CLOSED = 0;
	my $CLOSE_WAIT = 0;
	my $LAST_ACK = 0;
	my $CLOSING = 0;
	my $UNKNOWN = 0;

	while (<NETSTAT>) {
		chomp;
		if (/ESTABLISHED/) {
			$ESTABLISHED++;
		} elsif (/SYN_SENT/) {
			$SYN_SENT++;
		} elsif (/SYN_RECV/) {
			$SYN_RECV++;
		} elsif (/FIN_WAIT1/) {
			$FIN_WAIT1++;
		} elsif (/FIN_WAIT2/) {
			$FIN_WAIT2++;
		} elsif (/TIME_WAIT/) {
			$TIME_WAIT++;
		} elsif (/CLOSED/) {
			$CLOSED++;
		} elsif (/CLOSE_WAIT/) {
			$CLOSE_WAIT++;
		} elsif (/LAST_ACK/) {
			$LAST_ACK++;
		} elsif (/CLOSING/) {
			$CLOSING++;
		} elsif (/UNKNOWN/) {
			$UNKNOWN++;
		} 
	}

	close(NETSTAT);

	my $total = $ESTABLISHED + $SYN_SENT + $SYN_RECV + $FIN_WAIT1 + $FIN_WAIT2 + $TIME_WAIT + $CLOSED + $CLOSE_WAIT + $LAST_ACK + $CLOSING + $UNKNOWN;

	print "<metric type=\"IntCounter\" name=\"netstat:ESTABLISHED\" value=\"$ESTABLISHED\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:SYN_SENT\" value=\"$SYN_SENT\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:SYN_RECV\" value=\"$SYN_RECV\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:FIN_WAIT1\" value=\"$FIN_WAIT1\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:FIN_WAIT2\" value=\"$FIN_WAIT2\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:TIME_WAIT\" value=\"$TIME_WAIT\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:CLOSED\" value=\"$CLOSED\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:CLOSE_WAIT\" value=\"$CLOSE_WAIT\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:LAST_ACK\" value=\"$LAST_ACK\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:CLOSING\" value=\"$CLOSING\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:UNKNOWN\" value=\"$UNKNOWN\"/>\n";
	print "<metric type=\"IntCounter\" name=\"netstat:Total number of connections\" value=\"$total\"/>\n";

	print qq{<metric type="IntCounter" name="netstat:Success" value="2"/>\n};
};

if ( $@ ) {
	# write the error message to an error log for troubleshooting
	open(ERROR_LOG,">>$error_log") || die "Error when opening file \"$error_log\": $!";
	print ERROR_LOG localtime() . " - " . $@;
	close(ERROR_LOG);

	# gives the end user the opportunity to monitor the vmstat plugin
	print qq{<metric type="IntCounter" name="netstat:Success" value="1"/>\n};	
}
